/**
 * Stinger is an HTTP Request Validation Engine
 * Copyright (C) 2006  Aspect Security, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact us at info@aspectsecurity.com or www.aspectsecurity.com
 *
 */

package org.owasp.stinger;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.owasp.stinger.http.MutableHttpRequest;
import org.owasp.stinger.rules.RuleSet;
import org.owasp.stinger.actions.AbstractAction;

public class StingerFilter implements Filter {
	
	private Stinger stinger = null;
	
	private String config = null;

	private Boolean reload = false;
	
	private String errorPage = null;
	
	private ServletContext context = null;
	
	
	public void init(FilterConfig filterConfig) {
		
		/** Pull config location from Filter init parameter **/
		config = filterConfig.getInitParameter("config");
		/** Error page to display when exceptions are thrown **/
		errorPage = filterConfig.getInitParameter("error-page");
		/** Should we dynamically load the ruleset? **/
		reload = Boolean.valueOf(filterConfig.getInitParameter("reload"));
		/** Get the servlet context for Stinger **/
		context = filterConfig.getServletContext();
		/** Create a stinger instance **/
		stinger = new Stinger(new RuleSet(config), context);
	}
	
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) {
		MutableHttpRequest mRequest = null;
		HttpServletResponse hResponse = null;
		int retval = AbstractAction.CONTINUE;
		
		if(request instanceof HttpServletRequest && response instanceof HttpServletResponse) {
			mRequest = new MutableHttpRequest((HttpServletRequest)request);
			hResponse = (HttpServletResponse)response;
			
			if(reload) { stinger = new Stinger(new RuleSet(config), context); }
			
			try {
				retval = stinger.validate(mRequest, hResponse);
				
				switch(retval) {
					case AbstractAction.CONTINUE:
					case AbstractAction.PROCESS:
						chain.doFilter(mRequest, hResponse);
						break;
					case AbstractAction.DROP:
						break;
					default:
						break;
				}
			} catch(Exception e) {
				context.log("[Stinger-Filter] - " + e.getMessage(), e);
				
				try {
					hResponse.sendRedirect(errorPage);
				} catch (IOException ioe) {
					ioe.printStackTrace();
				}
			}
		}
	}
	
	public void destroy() {
		
	}
}
